@@ -1,25 +1,35 @@
-/*
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+/**
+ * xrdp: A Remote Desktop Protocol server.
+ *
+ * Copyright (C) Jay Sorg 2004-2014
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ *
+ * types
+ */
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+#ifndef _XRDP_TYPES_H_
+#define _XRDP_TYPES_H_
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#define DEFAULT_STRING_LEN 255
+#define LOG_WINDOW_CHAR_PER_LINE 60
 
-   xrdp: A Remote Desktop Protocol server.
-   Copyright (C) Jay Sorg 2004-2010
-
-   types
-
-*/
+#include "xrdp_rail.h"
+#include "xrdp_constants.h"
+#include "fifo.h"
 
+#define MAX_NR_CHANNELS 16
+#define MAX_CHANNEL_NAME 16
 /* lib */
 struct xrdp_mod
 {
@@ -32,13 +42,14 @@ struct xrdp_mod
                    long param3, long param4);
   int (*mod_signal)(struct xrdp_mod* v);
   int (*mod_end)(struct xrdp_mod* v);
-  int (*mod_set_param)(struct xrdp_mod* v, char* name, char* value);
+  int (*mod_set_param)(struct xrdp_mod* v, const char *name, char* value);
   int (*mod_session_change)(struct xrdp_mod* v, int, int);
   int (*mod_get_wait_objs)(struct xrdp_mod* v, tbus* read_objs, int* rcount,
                            tbus* write_objs, int* wcount, int* timeout);
   int (*mod_check_wait_objs)(struct xrdp_mod* v);
-  long mod_dumby[100 - 9]; /* align, 100 minus the number of mod 
-                              functions above */
+  int (*mod_frame_ack)(struct xrdp_mod* v, int flags, int frame_id);
+  tintptr mod_dumby[100 - 10]; /* align, 100 minus the number of mod
+                                  functions above */
   /* server functions */
   int (*server_begin_update)(struct xrdp_mod* v);
   int (*server_end_update)(struct xrdp_mod* v);
@@ -46,8 +57,10 @@ struct xrdp_mod
   int (*server_screen_blt)(struct xrdp_mod* v, int x, int y, int cx, int cy,
                            int srcx, int srcy);
   int (*server_paint_rect)(struct xrdp_mod* v, int x, int y, int cx, int cy,
-                           char* data, int width, int height, int srcx, int srcy);
-  int (*server_set_pointer)(struct xrdp_mod* v, int x, int y, char* data, char* mask);
+                           char* data, int width, int height,
+                           int srcx, int srcy);
+  int (*server_set_pointer)(struct xrdp_mod* v, int x, int y,
+                            char* data, char* mask);
   int (*server_palette)(struct xrdp_mod* v, int* palette);
   int (*server_msg)(struct xrdp_mod* v, char* msg, int code);
   int (*server_is_term)(struct xrdp_mod* v);
@@ -57,12 +70,12 @@ struct xrdp_mod
   int (*server_set_bgcolor)(struct xrdp_mod* v, int bgcolor);
   int (*server_set_opcode)(struct xrdp_mod* v, int opcode);
   int (*server_set_mixmode)(struct xrdp_mod* v, int mixmode);
-  int (*server_set_brush)(struct xrdp_mod* v, int x_orgin, int y_orgin,
+  int (*server_set_brush)(struct xrdp_mod* v, int x_origin, int y_origin,
                           int style, char* pattern);
   int (*server_set_pen)(struct xrdp_mod* v, int style,
                         int width);
   int (*server_draw_line)(struct xrdp_mod* v, int x1, int y1, int x2, int y2);
-  int (*server_add_char)(struct xrdp_mod* v, int font, int charactor,
+  int (*server_add_char)(struct xrdp_mod* v, int font, int character,
                          int offset, int baseline,
                          int width, int height, char* data);
   int (*server_draw_text)(struct xrdp_mod* v, int font,
@@ -75,18 +88,71 @@ struct xrdp_mod
   int (*server_query_channel)(struct xrdp_mod* v, int index,
                               char* channel_name,
                               int* channel_flags);
-  int (*server_get_channel_id)(struct xrdp_mod* v, char* name);
+  int (*server_get_channel_id)(struct xrdp_mod* v, const char *name);
   int (*server_send_to_channel)(struct xrdp_mod* v, int channel_id,
                                 char* data, int data_len,
                                 int total_data_len, int flags);
   int (*server_bell_trigger)(struct xrdp_mod* v);
-  long server_dumby[100 - 25]; /* align, 100 minus the number of server
-                                  functions above */
+  /* off screen bitmaps */
+  int (*server_create_os_surface)(struct xrdp_mod* v, int rdpindex,
+                                  int width, int height);
+  int (*server_switch_os_surface)(struct xrdp_mod* v, int rdpindex);
+  int (*server_delete_os_surface)(struct xrdp_mod* v, int rdpindex);
+  int (*server_paint_rect_os)(struct xrdp_mod* mod, int x, int y,
+                              int cx, int cy,
+                              int rdpindex, int srcx, int srcy);
+  int (*server_set_hints)(struct xrdp_mod* mod, int hints, int mask);
+  /* rail */
+  int (*server_window_new_update)(struct xrdp_mod* mod, int window_id,
+                                  struct rail_window_state_order* window_state,
+                                  int flags);
+  int (*server_window_delete)(struct xrdp_mod* mod, int window_id);
+  int (*server_window_icon)(struct xrdp_mod* mod,
+                            int window_id, int cache_entry, int cache_id,
+                            struct rail_icon_info* icon_info,
+                            int flags);
+  int (*server_window_cached_icon)(struct xrdp_mod* mod,
+                                   int window_id, int cache_entry,
+                                   int cache_id, int flags);
+  int (*server_notify_new_update)(struct xrdp_mod* mod,
+                                  int window_id, int notify_id,
+                                  struct rail_notify_state_order* notify_state,
+                                  int flags);
+  int (*server_notify_delete)(struct xrdp_mod* mod, int window_id,
+                              int notify_id);
+  int (*server_monitored_desktop)(struct xrdp_mod* mod,
+                                  struct rail_monitored_desktop_order* mdo,
+                                  int flags);
+  int (*server_set_pointer_ex)(struct xrdp_mod* v, int x, int y, char* data,
+                               char* mask, int bpp);
+  int (*server_add_char_alpha)(struct xrdp_mod* mod, int font, int character,
+                               int offset, int baseline,
+                               int width, int height, char* data);
+
+  int (*server_create_os_surface_bpp)(struct xrdp_mod* v, int rdpindex,
+                                      int width, int height, int bpp);
+  int (*server_paint_rect_bpp)(struct xrdp_mod* v, int x, int y, int cx, int cy,
+                               char* data, int width, int height,
+                               int srcx, int srcy, int bpp);
+  int (*server_composite)(struct xrdp_mod* v, int srcidx, int srcformat,
+                          int srcwidth, int srcrepeat, int* srctransform,
+                          int mskflags, int mskidx, int mskformat,
+                          int mskwidth, int mskrepeat, int op,
+                          int srcx, int srcy, int mskx, int msky,
+                          int dstx, int dsty, int width, int height,
+                          int dstformat);
+  int (*server_paint_rects)(struct xrdp_mod* v,
+                            int num_drects, short *drects,
+                            int num_crects, short *crects,
+                            char *data, int width, int height,
+                            int flags, int frame_id);
+  tintptr server_dumby[100 - 43]; /* align, 100 minus the number of server
+                                     functions above */
   /* common */
-  long handle; /* pointer to self as int */
-  long wm; /* struct xrdp_wm* */
-  long painter;
-  int sck;
+  tintptr handle; /* pointer to self as int */
+  tintptr wm; /* struct xrdp_wm* */
+  tintptr painter;
+  tintptr si;
 };
 
 /* header for bmp file */
@@ -114,6 +180,19 @@ struct xrdp_palette_item
 struct xrdp_bitmap_item
 {
   int stamp;
+  int lru_index;
+  struct xrdp_bitmap* bitmap;
+};
+
+struct xrdp_lru_item
+{
+  int next;
+  int prev;
+};
+
+struct xrdp_os_bitmap_item
+{
+  int id;
   struct xrdp_bitmap* bitmap;
 };
 
@@ -128,19 +207,23 @@ struct xrdp_pointer_item
   int stamp;
   int x; /* hotspot */
   int y;
-  char data[32 * 32 * 3];
+  char data[32 * 32 * 4];
   char mask[32 * 32 / 8];
+  int bpp;
 };
 
 struct xrdp_brush_item
 {
   int stamp;
   /* expand this to a structure to handle more complicated brushes
-     for now its 8x8 1bpp brushes only */
+     for now it's 8x8 1bpp brushes only */
   char pattern[8];
 };
 
-/* differnce caches */
+/* moved to xrdp_constants.h
+#define XRDP_BITMAP_CACHE_ENTRIES 2048 */
+
+/* difference caches */
 struct xrdp_cache
 {
   struct xrdp_wm* wm; /* owner */
@@ -152,6 +235,17 @@ struct xrdp_cache
   int bitmap_stamp;
   struct xrdp_bitmap_item bitmap_items[XRDP_MAX_BITMAP_CACHE_ID]
                                       [XRDP_MAX_BITMAP_CACHE_IDX];
+
+  /* lru optimize */
+  struct xrdp_lru_item bitmap_lrus[XRDP_MAX_BITMAP_CACHE_ID]
+                                  [XRDP_MAX_BITMAP_CACHE_IDX];
+  int lru_head[XRDP_MAX_BITMAP_CACHE_ID];
+  int lru_tail[XRDP_MAX_BITMAP_CACHE_ID];
+  int lru_reset[XRDP_MAX_BITMAP_CACHE_ID];
+
+  /* crc optimize */
+  struct list16 crc16[XRDP_MAX_BITMAP_CACHE_ID][64 * 1024];
+
   int use_bitmap_comp;
   int cache1_entries;
   int cache1_size;
@@ -170,8 +264,13 @@ struct xrdp_cache
   int pointer_cache_entries;
   int brush_stamp;
   struct xrdp_brush_item brush_items[64];
+  struct xrdp_os_bitmap_item os_bitmap_items[2000];
+  struct list* xrdp_os_del_list;
 };
 
+/* defined later */
+struct xrdp_enc_data;
+
 struct xrdp_mm
 {
   struct xrdp_wm* wm; /* owner */
@@ -187,11 +286,13 @@ struct xrdp_mm
   int (*mod_exit)(struct xrdp_mod*);
   struct xrdp_mod* mod; /* module interface */
   int display; /* 10 for :10.0, 11 for :11.0, etc */
-  int code; /* 0 Xvnc session 10 X11rdp session */
+  int code; /* 0=Xvnc session, 10=X11rdp session, 20=xorg driver mode */
   int sesman_controlled; /* true if this is a sesman session */
   struct trans* chan_trans; /* connection to chansrv */
   int chan_trans_up; /* true once connected to chansrv */
   int delete_chan_trans; /* boolean set when done with channel connection */
+  int usechansrv; /* true if chansrvport is set in xrdp.ini or using sesman */
+  struct xrdp_encoder *encoder;
 };
 
 struct xrdp_key_info
@@ -205,8 +306,11 @@ struct xrdp_keymap
   struct xrdp_key_info keys_noshift[256];
   struct xrdp_key_info keys_shift[256];
   struct xrdp_key_info keys_altgr[256];
+  struct xrdp_key_info keys_shiftaltgr[256];
   struct xrdp_key_info keys_capslock[256];
+  struct xrdp_key_info keys_capslockaltgr[256];
   struct xrdp_key_info keys_shiftcapslock[256];
+  struct xrdp_key_info keys_shiftcapslockaltgr[256];
 };
 
 /* the window manager */
@@ -266,6 +370,16 @@ struct xrdp_wm
   struct xrdp_mm* mm;
   struct xrdp_font* default_font;
   struct xrdp_keymap keymap;
+  int hide_log_window;
+  struct xrdp_bitmap* target_surface; /* either screen or os surface */
+  int current_surface_index;
+  int hints;
+  int allowedchannels[MAX_NR_CHANNELS];
+  int allowedinitialized ;
+  char pamerrortxt[256];
+
+  /* configuration derived from xrdp.ini */
+  struct xrdp_config *xrdp_config;
 };
 
 /* rdp process */
@@ -290,13 +404,14 @@ struct xrdp_listen
   struct trans* listen_trans; /* in tcp listen mode */
   struct list* process_list;
   tbus pro_done_event;
+  struct xrdp_startup_params* startup_params;
 };
 
 /* region */
 struct xrdp_region
 {
   struct xrdp_wm* wm; /* owner */
-  struct list* rects;
+  struct pixman_region16 *reg;
 };
 
 /* painter */
@@ -314,6 +429,9 @@ struct xrdp_painter
   struct xrdp_session* session;
   struct xrdp_wm* wm; /* owner */
   struct xrdp_font* font;
+  void *painter;
+  struct xrdp_region *dirty_region;
+  int begin_end_level;
 };
 
 /* window or bitmap */
@@ -366,7 +484,8 @@ struct xrdp_bitmap
   struct xrdp_bitmap* popped_from;
   int item_height;
   /* crc */
-  int crc;
+  int crc32;
+  int crc16;
 };
 
 #define NUM_FONTS 0x4e00
@@ -375,12 +494,12 @@ struct xrdp_bitmap
 #define DEFAULT_ELEMENT_TOP   35
 #define DEFAULT_BUTTON_W      60
 #define DEFAULT_BUTTON_H      23
-#define DEFAULT_COMBO_W       140
+#define DEFAULT_COMBO_W       210
 #define DEFAULT_COMBO_H       21
-#define DEFAULT_EDIT_W        140
+#define DEFAULT_EDIT_W        210
 #define DEFAULT_EDIT_H        21
-#define DEFAULT_WND_LOGIN_W   400
-#define DEFAULT_WND_LOGIN_H   200
+#define DEFAULT_WND_LOGIN_W   425
+#define DEFAULT_WND_LOGIN_H   475
 #define DEFAULT_WND_HELP_W    340
 #define DEFAULT_WND_HELP_H    300
 #define DEFAULT_WND_LOG_W     400
@@ -411,4 +530,89 @@ struct xrdp_startup_params
   int no_daemon;
   int help;
   int version;
+  int fork;
+  int send_buffer_bytes;
+  int recv_buffer_bytes;
 };
+
+/*
+ * For storing xrdp.ini configuration settings
+ */
+
+struct xrdp_cfg_globals
+{
+    int  ini_version;            /* xrdp.ini file version number */
+    int  use_bitmap_cache;
+    int  use_bitmap_compression;
+    int  port;
+    int  crypt_level;            /* low=1, medium=2, high=3 */
+    int  allow_channels;
+    int  max_bpp;
+    int  fork;
+    int  tcp_nodelay;
+    int  tcp_keepalive;
+    int  tcp_send_buffer_bytes;
+    int  tcp_recv_buffer_bytes;
+    char autorun[256];
+    int  hidelogwindow;
+    int  require_credentials;
+    int  bulk_compression;
+    int  new_cursors;
+    int  nego_sec_layer;
+    int  allow_multimon;
+
+    /* colors */
+
+    int  grey;
+    int  black;
+    int  dark_grey;
+    int  blue;
+    int  dark_blue;
+    int  white;
+    int  red;
+    int  green;
+    int  background;
+
+    /* login screen */
+    int  ls_top_window_bg_color; /* top level window background color */
+    int  ls_width;               /* window width */
+    int  ls_height;              /* window height */
+    int  ls_bg_color;            /* background color */
+    char ls_logo_filename[256];  /* logo filename */
+    char ls_background_image[256];  /* background image file name */
+    int  ls_logo_x_pos;          /* logo x co-ordinate */
+    int  ls_logo_y_pos;          /* logo y co-ordinate */
+    int  ls_label_x_pos;         /* x pos of labels */
+    int  ls_label_width;         /* width of labels */
+    int  ls_input_x_pos;         /* x pos of text and combo boxes */
+    int  ls_input_width;         /* width of input and combo boxes */
+    int  ls_input_y_pos;         /* y pos for for first label and combo box */
+    int  ls_btn_ok_x_pos;        /* x pos for OK button */
+    int  ls_btn_ok_y_pos;        /* y pos for OK button */
+    int  ls_btn_ok_width;        /* width of OK button */
+    int  ls_btn_ok_height;       /* height of OK button */
+    int  ls_btn_cancel_x_pos;    /* x pos for Cancel button */
+    int  ls_btn_cancel_y_pos;    /* y pos for Cancel button */
+    int  ls_btn_cancel_width;    /* width of Cancel button */
+    int  ls_btn_cancel_height;   /* height of Cancel button */
+    char ls_title[256];          /* loginscreen window title */
+};
+
+struct xrdp_cfg_logging
+{
+
+};
+
+struct xrdp_cfg_channels
+{
+
+};
+
+struct xrdp_config
+{
+    struct xrdp_cfg_globals   cfg_globals;
+    struct xrdp_cfg_logging   cfg_logging;
+    struct xrdp_cfg_channels  cfg_channels;
+};
+
+#endif
